"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMiddleware = void 0;
function createHasAccessMiddleware(client, getUserId, onAuthorizeFailure) {
    return (objectType, getObjectId, relation) => {
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            const objectId = getObjectId(req, res);
            const userId = getUserId(req, res);
            if (userId === null) {
                onAuthorizeFailure(req, res);
                return;
            }
            const hasAccess = yield client.isAuthorized({
                warrants: [{
                        objectType,
                        objectId,
                        relation,
                        subject: {
                            objectType: "user",
                            objectId: userId.toString(),
                        },
                    }],
            });
            if (hasAccess) {
                if (next)
                    next();
                return;
            }
            onAuthorizeFailure(req, res);
        });
    };
}
function createHasPermissionMiddleware(client, getUserId, onAuthorizeFailure) {
    return (permissionId) => {
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            const userId = getUserId(req, res);
            if (userId === null) {
                onAuthorizeFailure(req, res);
                return;
            }
            const hasPermission = yield client.hasPermission({
                permissionId,
                userId: userId.toString(),
            });
            if (hasPermission) {
                if (next)
                    next();
                return;
            }
            onAuthorizeFailure(req, res);
        });
    };
}
function createMiddleware(config) {
    let onAuthorizeFailure = (req, res) => res.sendStatus(401);
    if (config.onAuthorizeFailure) {
        onAuthorizeFailure = config.onAuthorizeFailure;
    }
    return {
        hasPermission: createHasPermissionMiddleware(config.client, config.getUserId, onAuthorizeFailure),
        hasAccess: createHasAccessMiddleware(config.client, config.getUserId, onAuthorizeFailure),
    };
}
exports.createMiddleware = createMiddleware;
